package cs636.pizza.config;

// In order to support both old fashioned JPA2 and Spring Data Repositories, we need a 
// Spring bean of type LocalContainerEntityManagerFactoryBean that the startup code for Spring Data
// will use to create an EntityManager that both approaches can use.
// Here we create such a bean, following https://www.baeldung.com/spring-data-jpa-multiple-databases
// (though he tackles the harder problem of using two databases at the same time)

import java.util.Map;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.orm.jpa.LocalContainerEntityManagerFactoryBean;
import org.springframework.orm.jpa.vendor.HibernateJpaVendorAdapter;
import org.springframework.transaction.annotation.EnableTransactionManagement;

@Configuration
@EnableJpaRepositories("cs636.pizza")
@EnableTransactionManagement
public class AppConfig {

	@Bean
	public LocalContainerEntityManagerFactoryBean entityManagerFactory() {  // needs this name to be found by JPA Repository	
		// This bean creation needs to be done during normal Spring bean creation time (tried to do it earlier but it didn't work)
		// So just collect the various properties earlier and do the following now--
		Map<String, String> emfProperties =  PizzaSystemConfig.getEntityManagerFactoryProperties();  // from start-up code
		// Special Spring bean builder that manages the JPA emf--
		LocalContainerEntityManagerFactoryBean em = new LocalContainerEntityManagerFactoryBean();
		em.setPersistenceUnitName("pizza2hib");
		em.setPackagesToScan(new String[] { "cs636.pizza" });
		HibernateJpaVendorAdapter vendorAdapter = new HibernateJpaVendorAdapter();
		em.setJpaVendorAdapter(vendorAdapter);
		Map<String, String> properties = emfProperties;
		em.setJpaPropertyMap(properties);
		// EntityManagerFactory emf = em.getObject();
		//PizzaSystemConfig.testJPA(emf);  // get null emf here, too early
		return em;
	}
	
	// code just for H2--
//	LocalContainerEntityManagerFactoryBean em = new LocalContainerEntityManagerFactoryBean();
	// em.setDataSource(ds);
//	em.setPersistenceUnitName("pizza2hib");
//	em.setPackagesToScan(new String[] { "cs636.pizza" });
//
//	HibernateJpaVendorAdapter vendorAdapter = new HibernateJpaVendorAdapter();
//	em.setJpaVendorAdapter(vendorAdapter);
//	HashMap<String, Object> properties = new HashMap<>();
//	properties.put("hibernate.dialect", "org.hibernate.dialect.H2Dialect");
//	properties.put("javax.persistence.jdbc.driver", "org.h2.Driver");
//	properties.put("javax.persistence.jdbc.url", "jdbc:h2:~/test");
//	properties.put("javax.persistence.jdbc.user", "test");
//	properties.put("javax.persistence.jdbc.password", "");
//	em.setJpaPropertyMap(properties);

}
