package cs636.pizza.dao;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import static cs636.pizza.dao.DBConstants.*;

@Repository
public class DbDAO {
	// the current transaction's em: it can clearly be a field here because this
	// app is single-threaded. When we turn this into a web app, we
	// have to be more careful, but in fact Spring uses a "facade",
	// an object that actually uses a ThreadLocal object at each point in
	// time, so that each thread gets its own em.
	@Autowired
	private EntityManager em;

	public EntityManager getEM() {
		return em;
	}
	
	public DbDAO(String dbUrl, String user, String password) {
	}

	// for DAO unit tests
	public DbDAO() {
	}

	public void initializeDb() {
		// drop tables with FK cols before the tables they refer to
		clearTable(PIZZA_TOPPING_TABLE);
		clearTable(ORDER_TABLE);
		clearTable(PIZZA_SIZE_TABLE);
		clearTable(MENU_TOPPING_TABLE);
		clearTable(MENU_SIZE_TABLE);
		clearTable(SYS_TABLE);
		clearTable(PIZZA_ID_GEN_TABLE);

		initSysTable();
		initIdGenTable();
	}

	// We can use direct SQL for DB setup easily as follows.
	// Any SQLException is handled by JPA, marking the
	// transaction as rollback-only, and then JPA throws a
	// javax.persistence.PersistenceException
	private int clearTable(String tableName) {
		Query q = em.createNativeQuery("delete from " + tableName);
		int n = q.executeUpdate(); // SQL of update shows in FINE logging
		// System.out.println("deleted " + n + " rows from " + tableName);
		return n;
	}

	// In pizza2, we're only using the last column (current_day) of this table
	// but we keep the others so we can run pizza1 on the same database

	private int initSysTable() {
		// System.out.println("inserting row (1,1,1,1,1,1) into " + SYS_TABLE);
		Query q = em.createNativeQuery("insert into " + SYS_TABLE + " values (1,1,1,1,1,1)");
		int n = q.executeUpdate();
		// System.out.println("inserted " + n + " rows into " + SYS_TABLE);
		return n;
	}

	// reinitialize id gen table so
	// as to get same sequence of ids on each re-init'd run
	private void initIdGenTable() {
		for (int i = 0; i < PIZZA_GEN_NAMES.length; i++)
			initIdGenTableOneRow(PIZZA_GEN_NAMES[i]);
	}

	private int initIdGenTableOneRow(String genName) {
		// System.out.println("inserting row (" + genName + ", 0) into " +
		// PIZZA_ID_GEN_TABLE);
		Query q = em.createNativeQuery("insert into " + PIZZA_ID_GEN_TABLE + " values ('" + genName + "', 0)");
		int n = q.executeUpdate();
		// System.out.println("inserted " + n + " rows into " + PIZZA_ID_GEN_TABLE);
		return n;
	}
	// Note no startTransaction, etc. needed: Spring takes care of that
}
